@@ -23,7 +23,19 @@ from typing import TYPE_CHECKING
 
 from tripper.errors import NamespaceError, TriplestoreError, UniquenessError
 from tripper.literal import Literal
-from tripper.namespace import DCTERMS, DM, FNO, MAP, OWL, RDF, RDFS, XML, XSD, Namespace
+from tripper.namespace import (
+    DCTERMS,
+    DM,
+    EMMO,
+    FNO,
+    MAP,
+    OWL,
+    RDF,
+    RDFS,
+    XML,
+    XSD,
+    Namespace,
+)
 from tripper.utils import en, function_id, infer_iri, split_iri
 
 if TYPE_CHECKING:  # pragma: no cover
@@ -379,57 +391,65 @@ class Triplestore:
             any: If true, return any matching value, otherwise raise
                 UniquenessError.
         """
-        triple = self.triples((subject, predicate, object))
+        spo = (subject, predicate, object)
+        if sum(iri is None for iri in spo) != 1:
+            raise ValueError(
+                "Exactly one of `subject`, `predicate` or `object` must be None."
+            )
+
+        triple = self.triples(spo)
         try:
             value = next(triple)
         except StopIteration:
             return default
 
-        if any:
-            return value
+        # Index of subject-predicate-object argument that is None
+        (idx,) = [i for i, v in enumerate(spo) if v is None]
 
         try:
             next(triple)
         except StopIteration:
-            return value
-        else:
-            raise UniquenessError("More than one match")
+            return value[idx]
+
+        if any:
+            return value[idx]
+        raise UniquenessError("More than one match")
 
     def subjects(
         self, predicate=None, object=None  # pylint: disable=redefined-builtin
     ):
         """Returns a generator of subjects for given predicate and object."""
-        for s, _, _ in self.triples((None, predicate, object)):
+        for s, _, _ in self.triples(predicate=predicate, object=object):
             yield s
 
     def predicates(
         self, subject=None, object=None  # pylint: disable=redefined-builtin
     ):
         """Returns a generator of predicates for given subject and object."""
-        for _, p, _ in self.triples((subject, None, object)):
+        for _, p, _ in self.triples(subject=subject, object=object):
             yield p
 
     def objects(self, subject=None, predicate=None):
         """Returns a generator of objects for given subject and predicate."""
-        for _, _, o in self.triples((subject, predicate, None)):
+        for _, _, o in self.triples(subject=subject, predicate=predicate):
             yield o
 
     def subject_predicates(self, object=None):  # pylint: disable=redefined-builtin
         """Returns a generator of (subject, predicate) tuples for given
         object."""
-        for s, p, _ in self.triples((None, None, object)):
+        for s, p, _ in self.triples(object=object):
             yield s, p
 
     def subject_objects(self, predicate=None):
         """Returns a generator of (subject, object) tuples for given
         predicate."""
-        for s, _, o in self.triples((None, predicate, None)):
+        for s, _, o in self.triples(predicate=predicate):
             yield s, o
 
     def predicate_objects(self, subject=None):
         """Returns a generator of (predicate, object) tuples for given
         subject."""
-        for _, p, o in self.triples((subject, None, None)):
+        for _, p, o in self.triples(subject=subject):
             yield p, o
 
     def set(self, triple):
@@ -447,7 +467,7 @@ class Triplestore:
     ):  # pylint: disable=redefined-builtin
         """Returns true if the triplestore has any triple matching
         the give subject, predicate and/or object."""
-        triple = self.triples((subject, predicate, object))
+        triple = self.triples(subject=subject, predicate=predicate, object=object)
         try:
             next(triple)
         except StopIteration:
@@ -483,6 +503,32 @@ class Triplestore:
                 raise NamespaceError(f"No prefix defined for IRI: {iri}")
         return iri
 
+    def map(
+        self,
+        source: str,
+        target: str,
+        cost: "Optional[Union[float, Callable]]" = None,
+        target_cost: bool = True,
+    ):
+        """Add 'mapsTo' relation to the triplestore.
+
+        Parameters:
+            source: Source IRI.
+            target: IRI of target ontological concept.
+            cost: User-defined cost of following this mapping relation
+                represented as a float.  It may be given either as a
+                float or as a callable taking the value of the mapped
+                quantity as input and returning the cost as a float.
+            target_cost: Whether the cost is assigned to mapping steps
+                that have `target` as output.
+        """
+        return self.add_mapsTo(
+            target=target,
+            source=source,
+            cost=cost,
+            target_cost=target_cost,
+        )
+
     def add_mapsTo(
         self,
         target: str,
@@ -504,6 +550,10 @@ class Triplestore:
                 quantity as input and returning the cost as a float.
             target_cost: Whether the cost is assigned to mapping steps
                 that have `target` as output.
+
+        Note:
+            This is equivalent to the `map()` method, but reverts the
+            two first arguments and adds the `property_name` argument.
         """
         self.bind("map", MAP)
 
@@ -528,7 +578,7 @@ class Triplestore:
         expects: "Union[str, Sequence, Mapping]" = (),
         returns: "Union[str, Sequence]" = (),
         base_iri: "Optional[str]" = None,
-        standard: str = "fno",
+        standard: str = "emmo",
         cost: "Optional[Union[float, Callable]]" = None,
     ):
         """Inspect function and add triples describing it to the triplestore.
@@ -544,7 +594,9 @@ class Triplestore:
             base_iri: Base of the IRI representing the function in the
                 knowledge base.  Defaults to the base IRI of the triplestore.
             standard: Name of ontology to use when describing the function.
-                Defaults to the Function Ontology (FnO).
+                Valid values are:
+                - "emmo": Elementary Multiperspective Material Ontology (EMMO)
+                - "fno": Function Ontology (FnO)
             cost: User-defined cost of following this mapping relation
                 represented as a float.  It may be given either as a
                 float or as a callable taking the same arguments as `func`
@@ -588,6 +640,7 @@ class Triplestore:
 
     def _add_function_fno(self, func, expects, returns, base_iri):
         """Implementing add_function() for FnO."""
+        # pylint: disable=too-many-locals,too-many-statements
         self.bind("fno", FNO)
         self.bind("dcterms", DCTERMS)
         self.bind("map", MAP)
@@ -598,6 +651,7 @@ class Triplestore:
         if callable(func):
             fid = function_id(func)  # Function id
             func_iri = f"{base_iri}{func.__name__}_{fid}"
+            name = func.__name__
             doc_string = inspect.getdoc(func)
             parlist = f"_:{func.__name__}{fid}_parlist"
             outlist = f"_:{func.__name__}{fid}_outlist"
@@ -609,6 +663,7 @@ class Triplestore:
                 ]
         elif isinstance(func, str):
             func_iri = func
+            name = split_iri(func)[1]
             doc_string = ""
             parlist = f"_:{func_iri}_parlist"
             outlist = f"_:{func_iri}_outlist"
@@ -619,6 +674,7 @@ class Triplestore:
             raise TypeError("`func` should be either a callable or an IRI")
 
         self.add((func_iri, RDF.type, FNO.Function))
+        self.add((func_iri, RDFS.label, en(name)))
         self.add((func_iri, FNO.expects, parlist))
         self.add((func_iri, FNO.returns, outlist))
         if doc_string:
@@ -646,3 +702,70 @@ class Triplestore:
             lst = lst_next
 
         return func_iri
+
+    def _add_function_emmo(self, func, expects, returns, base_iri):
+        """Implementing add_function() method for the "emmo" standard."""
+        # pylint: disable=too-many-locals
+        self.bind("emmo", EMMO)
+        self.bind("dcterms", DCTERMS)
+        self.bind("map", MAP)
+
+        # Hardcode EMMO IRIs to avoid slow lookup
+        Task = EMMO.EMMO_4299e344_a321_4ef2_a744_bacfcce80afc
+        DataSet = EMMO.EMMO_194e367c_9783_4bf5_96d0_9ad597d48d9a
+        hasInput = EMMO.EMMO_36e69413_8c59_4799_946c_10b05d266e22
+        hasOutput = EMMO.EMMO_c4bace1d_4db0_4cd3_87e9_18122bae2840
+        # Software = EMMO.EMMO_8681074a_e225_4e38_b586_e85b0f43ce38
+        # hasSoftware = EMMO.Software  # TODO: fix when EMMO has hasSoftware
+
+        if base_iri is None:
+            base_iri = self.base_iri if self.base_iri else ":"
+
+        if callable(func):
+            fid = function_id(func)  # Function id
+            func_iri = f"{base_iri}{func.__name__}_{fid}"
+            name = func.__name__
+            doc_string = inspect.getdoc(func)
+            if isinstance(expects, Sequence):
+                pars = list(zip(inspect.signature(func).parameters, expects))
+            else:
+                pars = expects.items()
+        elif isinstance(func, str):
+            func_iri = func
+            name = split_iri(func)[1]
+            doc_string = ""
+            pariris = expects if isinstance(expects, Sequence) else expects.values()
+            parnames = [split_iri(pariri)[1] for pariri in pariris]
+            pars = list(zip(parnames, pariris))
+        else:
+            raise TypeError("`func` should be either a callable or an IRI")
+
+        self.add((func_iri, RDF.type, Task))
+        self.add((func_iri, RDFS.label, en(name)))
+        for parname, iri in pars:
+            self.add((iri, RDF.type, DataSet))
+            self.add((iri, RDFS.label, en(parname)))
+            self.add((func_iri, hasInput, iri))
+        for iri in returns:
+            self.add((iri, RDF.type, DataSet))
+            self.add((func_iri, hasOutput, iri))
+        if doc_string:
+            self.add((func_iri, DCTERMS.description, en(doc_string)))
+
+        # self.add((func_iri, RDF.type, Task))
+        # self.add((func_iri, RDFS.label, en(name)))
+        # for parname, iri in pars:
+        #    par = f"{func_iri}_input_{parname}"
+        #    self.add((par, RDF.type, DataSet))
+        #    self.add((par, RDFS.label, en(parname)))
+        #    self.add((par, MAP.mapsTo, iri))
+        #    self.add((func_iri, hasInput, par))
+        # for i, iri in enumerate(returns):
+        #    val = f"{func_iri}_output{i+1}"
+        #    self.add((val, RDF.type, DataSet))
+        #    self.add((val, MAP.mapsTo, iri))
+        #    self.add((func_iri, hasOutput, val))
+        # if doc_string:
+        #    self.add((func_iri, DCTERMS.description, en(doc_string)))
+
+        return func_iri
